﻿using System;
using System.Text;

namespace mPCIe_DIO_24s
{    
    /// <summary>
    /// Enumerations for return status
    /// </summary>
    public enum eStatus
    {
        eSuccess = 0,
        eFailure = -1,
    };

    /// <summary>
    /// Enumeration of Available ports
    /// </summary>
    public enum ePortName
    {
        ePortA = 0,
        ePortB,
        ePortCL,
        ePortCH,
        ePortUnknown
    };

    /// <summary>
    /// /// Enumeration of Available pins
    /// </summary>
    public enum ePinNum
    {
        ePin0 = 0,
        ePin1,
        ePin2,
        ePin3,
        ePin4,
        ePin5,
        ePin6,
        ePin7,
        ePinUnknown
    };

    /// <summary>
    /// Enumeration for Direction
    /// </summary>
    public enum eDirection
    {
        eInput = 0,
        eOutput,
        eUnknown
    };

    /// <summary>
    /// Enumeration for Supported Alternate functions
    /// </summary>
    public enum eAltFunction
    {
        // Input Alternate functions
        eGenIn = 0,
        ePulseDetect,
        eCntPulse,
        eCntQuadPulse,
        eMeasPulseHigh,
        eMeasFreq,
        eMeasDutyCycle,
        // Output Alternate functions
        eGenOut,
        eGenPulse,
        eGenPulseTrain,
        eGenDutyCycle,
        eGenQuad
    }

    /// <summary>
    /// Supported configuration paramters
    /// </summary>    
    public enum eConfigParam
    {
        eGenOutput = 0,      
        eGenInput,    
        eRisingEdge,
        eFallingEdge,
        eLowPulse,
        eHighPulse, 
        eFreqMeasure,
        ePulseDirHigh,
        ePulseDirLow, 
        eDirForward,
        eDirReverse,      
        eNone
    };

    /// <summary>
    /// Structure Detect Pulse Configuration parameter
    /// </summary>
    public struct DetectPulseConfig
    {
        public eConfigParam trigger_type;
        public bool enable_irq;
    }

    /// <summary>
    /// Structure Pulse Train Configuration parameter
    /// </summary>
    public struct PulseTrainConfig
    {
        public eConfigParam pulse_dir;
        public float pulse_len_msec;
        public int pulse_cnt;
        public float time_bw_pulses_msec;
    }

    /// <summary>
    /// Structure Duty cycle Configuration paramter
    /// </summary>
    public struct DutyCycleConfig
    {
        public int time_period;
        public int dutycycle;
    }

    /// <summary>
    /// Structure for all configuration paramters
    /// </summary>
    public struct updateConfigParam
    {
        public ePortName pName;
        public ePinNum pNum;
        public eAltFunction altFun;
        public DetectPulseConfig detectPulse_config;
        public PulseTrainConfig pulseTrain_config;
        public DutyCycleConfig dutyCycle_config;
    }
}
/***************************************************************************************************************/
/* End of File                                                                                                 */
/***************************************************************************************************************/