﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using mPCIe_DIO_24s;
using System.Threading;

using AIOWDMNet;  // the namespace exposes the AIOWDM Class interface 

namespace mPCIe_DIO_24s
{    
    public partial class Main_Form : Form
    {
        // Defined constants
        public const int DEVICE_ID = 0x0E57;

        // This will hold the data for the card installed in the system:
        public struct TCardData
        {
            public bool IsValid;
            public bool IsSelected;
            public UInt32 DeviceID;
            public UInt32 Base;     // This a result of findcards and the base address of the card
        };

        /// <summary>
        /// Static variable declarations 
        /// </summary>
        // Card details
        static UInt32 Status = 0;
        static Int32 NumCards = 0;
        static Int32 CardNum = 0;   // The index of the one card we will be using 0
        static UInt32 Offset = 0;   // This the offset that will be used based on the base address of the card
        static public TCardData CardData; // only one struct

        // DIO offsets +4 each decimal 0,4,8,12,16
        static UInt32[] PortOffsets = new UInt32[10] { 0x0, 0x4, 0x8, 0xB, 0x10, 0x14, 0x18, 0x1B, 0x20, 0x24 };                 
                
        /* IRQ thread handler*/
        static Thread irq_thread_handler = null;

        /// <Main_Form>
        /// Param:  None
        /// Return: None
        /// </summary>
        public Main_Form()
        {
            InitializeComponent();            
        }

        /// <Finds the connected card details>
        /// Param:      None
        /// Return:     None
        /// </summary>
        private void FindCards()
        {            
            // Local Vars for QueryCardInfo():
            UInt32 DeviceID = 0, Base;
            UInt32 NameSize = 256;
            UInt16[] Name = new UInt16[256];
            String strname;

            // Set CardData to 0 as needed:
            CardData.IsValid = false;
            CardData.IsSelected = false;
            CardData.DeviceID = 0;
            CardData.Base = 0;

            // Get the total number of cards installed:
            NumCards = AIOWDM.GetNumCards();

            if (NumCards == 0) // no cards present
	        {
                statusStrip.Text = "No Card Found, NumCards = 0";
                tbLog.Text = "No cards were found!   \r\n";
                tbLog.Text += "This may mean the card is not installed.  \r\n";
                tbLog.Text += "Check Device Manager for a card and its status.  \r\n";
                tbLog.Text += "You may consider rebooting your system.  ";
                lbDeviceInfo.Text = "Unknown Device";
                toolStripStatusLabel1.Text = "Device not Connected";
	        }
	        else 
            {
                // Validate and store card data:                
                Status = AIOWDM.QueryCardInfo(CardNum, out DeviceID, out Base, out NameSize, out strname);
                MessageBox.Show("status: " + DeviceID);
                switch (DeviceID)
                {           
                            
                    case DEVICE_ID:
                        lbDeviceInfo.Text = "mPCIe-DIO-24S Parallel Digital I/O Card";
                        toolStripStatusLabel1.Text = strname + "Connected";
                        CardData.IsValid = true;
                        CardData.DeviceID = DeviceID;
                        CardData.Base = Base;                                                
                        break;

                    default:
                        lbDeviceInfo.Text = "Unknown Device";
                        toolStripStatusLabel1.Text = "Unknown Device";
                        break;
                }// end switch			

            }//end else card present
        }               

        /// <summary>
        /// Function called while loading the main form
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Main_Form_Load(object sender, EventArgs e)
        {                     
            // Add direction elements to direction combobox
            string[] direction_string = new string[] {"Input", "Output"};            
            cbDirPortA.Items.AddRange(direction_string);
            cbDirPortB.Items.AddRange(direction_string);
            cbDirPortCL.Items.AddRange(direction_string);
            cbDirPortCH.Items.AddRange(direction_string);

            // By Default all the ports are input port 
            cbDirPortA.SelectedIndex = 0;
            cbDirPortB.SelectedIndex = 0;
            cbDirPortCL.SelectedIndex = 0;
            cbDirPortCH.SelectedIndex = 0;

            /* Add user controls to the main UI */
            UserControlAdd(ePortName.ePortA);
            UserControlAdd(ePortName.ePortB);
            UserControlAdd(ePortName.ePortCL);
            UserControlAdd(ePortName.ePortCH);

            // Initialize the user control with all inputports                            
            UserControlInit(fLytPanPortA);
            UserControlInit(fLytPanPortB);
            UserControlInit(fLytPanPortCL);
            UserControlInit(fLytPanPortCH);

            /* Find and update mPCIe Card info */
            FindCards();

            // Create & Start the thread to handle IRQ's
            irq_thread_handler = new Thread(new ThreadStart(HandleIrqThread));
            irq_thread_handler.Start();
        }

        /// <summary>
        /// Function get invoked while closing the Main form
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Main_Form_FormClosing(object sender, FormClosingEventArgs e)
        {
            // Abort the thread
            irq_thread_handler.Abort();
        }

        /// <summary>
        /// Addes the user controls to hte flow layout panel
        /// </summary>
        /// <param name="port_name">enumeration represents port name</param>
        private void UserControlAdd(ePortName port_name)
        {
            // Dynamically add Usercontrol to Port A 
            for (int i = 0; i < 8; i++)
            {
                ucPinConfig uc = new ucPinConfig();
                uc.PinNumber = i;
                if (port_name == ePortName.ePortA)
                {
                    uc.PortName = 'A';
                    uc.Name = "A" + i;
                    fLytPanPortA.Controls.Add(uc);
                }
                else if (port_name == ePortName.ePortB)
                {
                    uc.PortName = 'B';
                    uc.Name = "B" + i;
                    fLytPanPortB.Controls.Add(uc);
                }
                else if ((port_name == ePortName.ePortCL) && (i < 4))
                {
                    uc.PortName = 'C';
                    uc.Name = "C" + i;
                    fLytPanPortCL.Controls.Add(uc);
                }
                else if ((port_name == ePortName.ePortCH) && (i > 3))
                {
                    uc.PortName = 'C';
                    uc.Name = "C" + i;
                    fLytPanPortCH.Controls.Add(uc);
                }
            }
        }

        /// <summary>
        /// Initiaize the user control with default values
        /// </summary>
        /// <param name="flyLytPan">control to the FlowLayoutPanel</param>
        private void UserControlInit(FlowLayoutPanel flwLytPan)
        {
            eDirection selectedIndex = eDirection.eInput;
            ucPinConfig pin_config_UC = null;
            try
            {
                // Invoke an anonymous method on the thread of the form.
                MethodInvoker mi = delegate
                {
                    // Update Port - A user controls

                    foreach (Control ctl in flwLytPan.Controls)
                    {
                        if (ctl is ucPinConfig)
                        {
                            pin_config_UC = (ctl as ucPinConfig);
                            pin_config_UC.UpdatePinAlternateFunction(selectedIndex);
                        }
                    }
                };
                this.Invoke(mi);
            }
            catch
            {
                MessageBox.Show("ERROR: While addeing usercontrol to the main UI");
            }
        }

        /// <summary>
        /// Thread handles the IRQ's
        /// </summary>
        public void HandleIrqThread()
        {
            for (;;)
            {
                if (AIOWDM.WaitForIRQ(CardNum) > 0)
                {
                    //TODO: Handle IRQ's
                    MessageBox.Show("Interrupt generated");
                }
            }
        }

        /// <summary>
        /// Returns the system date and time in string format
        /// </summary>
        /// <returns>String containing current date and time </returns>
        private string GetTimeStamp()
        {
            string time_stamp = DateTime.Now.ToString("yyyy:MM:dd HH:mm:ss - ");
            return time_stamp;             
        }

        /// <summary>
        /// Updates the support alternate function options based on the set direction 
        /// </summary>
        /// <param name="flowLytPan">handle for FlowLayoutPanel</param>
        /// <param name="dir">direction in or out </param>
        private void UpdateAlternateFunctions(FlowLayoutPanel flowLytPan, eDirection dir)
        {
            ucPinConfig pin_config_UC = null;

            /* Update the combobox based on the direction set */
            try
            {
                // Invoke an anonymous method on the thread of the form.
                MethodInvoker mi = delegate
                {
                    foreach (Control ctl in flowLytPan.Controls)
                    {
                        if (ctl is ucPinConfig)
                        {
                            pin_config_UC = (ctl as ucPinConfig);
                            pin_config_UC.UpdatePinAlternateFunction(dir);
                        }
                    }
                };
                this.Invoke(mi);
            }
            catch { }            
        }

        /// <summary>
        /// Handler for port-A direction change event
        /// Function updates the Alternate functions for input & ouput ports
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void cbDirPortA_SelectedIndexChanged(object sender, EventArgs e)
        {
            ComboBox cmb = (ComboBox)sender;
            UpdateAlternateFunctions(fLytPanPortA, (eDirection)cmb.SelectedIndex);
        }

        /// <summary>
        /// Handler for port-B direction change event
        /// Function updates the Alternate functions for input & ouput ports
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void cbDirPortB_SelectedIndexChanged(object sender, EventArgs e)
        {
            ComboBox cmb = (ComboBox)sender;
            UpdateAlternateFunctions(fLytPanPortB, (eDirection)cmb.SelectedIndex);
        }

        /// <summary>
        /// Handler for port-CL direction change event
        /// Function updates the Alternate functions for input & ouput ports
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void cbDirPortCL_SelectedIndexChanged(object sender, EventArgs e)
        {
            ComboBox cmb = (ComboBox)sender;
            UpdateAlternateFunctions(fLytPanPortCL, (eDirection)cmb.SelectedIndex);
        }

        /// <summary>
        /// Handler for port-CH direction change event
        /// Function updates the Alternate functions for input & ouput ports
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void cbDirPortCH_SelectedIndexChanged(object sender, EventArgs e)
        {
            ComboBox cmb = (ComboBox)sender;
            UpdateAlternateFunctions(fLytPanPortCH, (eDirection)cmb.SelectedIndex);
        }

        /// <summary>
        /// Handler for Clear Log button click event - Clears the log text
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btClearLog_Click(object sender, EventArgs e)
        {
            DialogResult dialogResult = MessageBox.Show("This will clear the log data. Do you want to continue?", "Warning", MessageBoxButtons.YesNo);
            if (dialogResult == DialogResult.Yes)
            {
                // Clear the text                
                tbLog.Clear();
            }            
        }

        /// <summary>
        /// Handler for Automode checkbox change event
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void chkbxEnableAutomode_CheckedChanged(object sender, EventArgs e)
        {
            if (chkbxEnableAutomode.Checked == true)
            {                
                // Enable Update & Run Button
                btUpdate.Enabled = true;
                btRun.Enabled = true;
            }
            else
            {
                // Disable Update & Run Button, valid only in Auto mode
                btUpdate.Enabled = false;
                btRun.Enabled = false;
            }
        }

        /// <summary>
        /// Handler for Update button click event
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btUpdate_Click(object sender, EventArgs e)
        {                        
            //TODO: Update the register values

            MessageBox.Show("This will update the values to registers", "TODO");            
        }

        /// <summary>
        /// Handler for Run Button Click event
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btRun_Click(object sender, EventArgs e)
        {
            //TODO: Set the control bit value

            MessageBox.Show("This will Set the control bit to execute loaded values", "TODO");            
        }

        /// <summary>
        /// Periodic Timer of interval 100mSec
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>        
        private void Periodic_timer_Tick(object sender, EventArgs e)
        {
            /* TODO: Read All Input port status & Update the status on UI */
            UInt16 read_val = AIOWDM.RelInPort(CardNum, Offset);
        }

        /// <summary>
        /// Clear Counters button click event handler
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btClearCounter_Click(object sender, EventArgs e)
        {
            DialogResult dialogResult = MessageBox.Show("This will reset all the counter values. Do you want to continue?", "Warning", MessageBoxButtons.YesNo);
        }

        /// <summary>
        /// Function to update the Configuration paramter to registers
        /// </summary>
        /// <param name="configParam">Configuration paramters</param>
        public static void UpdatePinConfig(updateConfigParam configParam)
        {
            /* TODO: Convert ConfigParam to regiter value and write to appropriate registers*/
            switch (configParam.altFun)
            {
                case eAltFunction.ePulseDetect:
                    //TODO
                    break;
                case eAltFunction.eGenPulseTrain:
                    //TODO
                    break;
                case eAltFunction.eMeasDutyCycle:
                    //TODO
                    break;
                default:
                    break;
            }            
        }      

        // TODO: Remove data, only for testing purpose
        static byte data = 0x00;
        /// <summary>
        /// Update the ouput state based on the param
        /// </summary>
        /// <param name="portName">Port name</param>
        /// <param name="pinName">Pin Number</param>
        /// <param name="value">Value to be updated</param>
        /// <returns>Status</returns>
        public static eStatus UpdateOutputState(ePortName portName, ePinNum pinNum, bool value)
        {
            eStatus status = eStatus.eFailure;

            /* TODO
             * 1. Read the register value
             * 2. Change the perticular bit 
             * 3. Write to Register
             * 4. Read back and confirm write success
             * 5. Return the status based on step-4
             */

            /* <<Testing purpose only>> */
            if (data == 0)
                data = 0xFF;
            else
                data = 0x00;

            Offset = PortOffsets[0];
            ushort retval = AIOWDM.RelOutPort(CardNum, Offset + 0, data);
            UInt16 temp = AIOWDM.RelInPort(CardNum, Offset);
            if (data == temp)
            {
                status = eStatus.eSuccess;
            }
            MessageBox.Show("Return value from RelOutPort = 0x" + retval.ToString("X"));
            /* <<End of test>> */

            return status;
        }

        /// <summary>
        /// Updates the inputput parameters on the User interface
        /// </summary>
        /// <param name="flowLytPan"></param>
        /// <param name="pinNum"></param>
        /// <param name="param"></param>
        /// <param name="data"></param>
        private void UpdateInputParams(FlowLayoutPanel flowLytPan, ePinNum pinNum, eConfigParam param, UInt32 data)
        {
            ucPinConfig ip_pin_val = null;

            // Invoke an anonymous method on the thread of the form.
            MethodInvoker mi = delegate
            {
                foreach (Control ctl in flowLytPan.Controls)
                {
                    if (ctl is ucPinConfig)
                    {
                        ip_pin_val = (ctl as ucPinConfig);
                        if (pinNum == (ePinNum)ip_pin_val.PinNumber)
                        {
                            ip_pin_val.UpdateInputValue(param, data);
                        }
                    }
                }                
            };
            this.Invoke(mi);
        }

        /// <summary>
        /// Function updates the General input status on the UI
        /// </summary>
        /// <param name="portName">Port Name</param>
        /// <param name="pinNum">Port Number</param>
        /// <param name="param">Input Parameter to be updated</param>
        /// <param name="data">Input data to be updated</param>
        public void UpdateInputState(ePortName portName, ePinNum pinNum, eConfigParam param, UInt32 data)
        {
            switch (portName)
            {
                case ePortName.ePortA:
                    UpdateInputParams(fLytPanPortA, pinNum, param, data);
                    break;
                case ePortName.ePortB:
                    UpdateInputParams(fLytPanPortB, pinNum, param, data);
                    break;
                case ePortName.ePortCL:
                    UpdateInputParams(fLytPanPortCL, pinNum, param, data);
                    break;
                case ePortName.ePortCH:
                    UpdateInputParams(fLytPanPortCH, pinNum, param, data);
                    break;
                default:
                    break;
            }
        }

        /// <summary>
        /// Only for testing purpose will not inlcude in final release
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btTestButton_Click(object sender, EventArgs e)
        {
            /*@NOTE: only for testing purpose */
            UpdateInputState(ePortName.ePortA, ePinNum.ePin2, eConfigParam.eGenInput, 0);
            UpdateOutputState(ePortName.ePortA, ePinNum.ePin0, true);
        }
    }
}
/***************************************************************************************************************/
/* End of File                                                                                                 */
/***************************************************************************************************************/