﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace mPCIe_DIO_24s
{
    public partial class ucPinConfig : UserControl
    {                
        static string last_changed_pin_name = null;

        private int m_PinNumber;
        private int alt_fun;
        private char m_PortName;        

        /* String constants */
        const string s_gen_op = "General Output";
        const string s_genSigPul_op = "Generate Single Pulse";
        const string s_genPulTrn_op = "Generarte Pulse Train";
        const string s_genPWM_op = "Generate PWM";
        const string s_genMot_op = "Motor Control Output";        
        
        const string s_gen_ip = "General Input";        
        const string s_pDetect_ip = "Detect Pulse (Event)";
        const string s_EvntCntr_ip = "Event Counter";
        const string s_QPulCntr_ip = "Quadrature Pulse Counter";
        const string s_PWHMeas_ip = "Measure Pulse Width";
        const string s_FreqMeas_ip = "Measure Frequency";
        const string s_PWMMeas_ip = "Measure Duty Cycle";

        public static eConfigParam trigger_type = eConfigParam.eNone;
        
        /// <summary>
        /// Set or get the port name property
        /// </summary>
        public int PortName
        {
            get
            {
                return m_PortName;
            }
            set
            {
                m_PortName = (char)value;                
                lbPinName.Text = "P" + m_PortName + "[" + m_PinNumber + "]";
            }
        }

        /// <summary>
        /// Set or get the Pin Number property
        /// </summary>
        public int PinNumber
        {
            get
            {
                return m_PinNumber;
            }
            set
            {
                m_PinNumber = value;
                lbPinName.Text = "P"+ m_PortName +"[" + m_PinNumber + "]";
            }
        }

        /// <summary>
        /// Set or get the Alternate Function type
        /// </summary>
        public int AltFunction
        {
            get
            {
                return alt_fun;
            }
            set
            {
                alt_fun = value;                
            }            
        }        

        /// <summary>
        /// Initialize the uiPinConfig controls
        /// </summary>
        public ucPinConfig()
        {
            InitializeComponent();                         
        }
        
        /// <summary>
        /// Returns port name and pin from the port name string 
        /// </summary>
        /// <returns>Port Name & Pin Number</returns>
        public static Tuple<ePortName, ePinNum> Get_PortPin_Name()
        {
            ePortName portName = ePortName.ePortUnknown;
            ePinNum pinName = ePinNum.ePinUnknown ;

            for (int i = 0; i < 24; i++)
            {
                if (last_changed_pin_name == ("A" + i))
                {                    
                    portName = ePortName.ePortA;
                    pinName = (ePinNum)i;
                    break;
                }
                if (last_changed_pin_name == ("B" + i))
                {
                    portName = ePortName.ePortB;
                    pinName = (ePinNum)i;
                    break;
                }
                if (last_changed_pin_name == ("C" + i))
                {
                    if (i <= 3)
                    {
                        portName = ePortName.ePortCL;
                        pinName = (ePinNum)i;
                    }
                    else
                    {
                        portName = ePortName.ePortCH;
                        pinName = (ePinNum)i;
                    }
                    break;
                }
            }
            return new Tuple<ePortName, ePinNum>(portName, pinName);
        }
               
        /// <summary>
        /// Pulse detect configuration update callback
        /// </summary>
        /// <param name="trigger_type"></param>
        /// <param name="irq"></param>
        public static void PulseDetectConfigUpdate(eConfigParam trigger_type, bool irq)
        {
            updateConfigParam configParam = new updateConfigParam();
            Tuple<ePortName, ePinNum> pin;
            pin = Get_PortPin_Name();     
            
            configParam.pName = (ePortName)pin.Item1;
            configParam.pNum = (ePinNum)pin.Item2;
            configParam.altFun = eAltFunction.ePulseDetect;
            configParam.detectPulse_config.trigger_type = trigger_type;
            configParam.detectPulse_config.enable_irq = irq;
            Main_Form.UpdatePinConfig(configParam);            
        }

        public static void PulseTrainConfigUpdate (eConfigParam pulse_dir, float pulse_len_msec, int pulse_cnt, float time_bw_pulses_msec)
        {
            updateConfigParam configParam = new updateConfigParam();
            Tuple<ePortName, ePinNum> pin;
            pin = Get_PortPin_Name();

            configParam.pName = (ePortName)pin.Item1;
            configParam.pNum = (ePinNum)pin.Item2;
            configParam.altFun = eAltFunction.eGenPulseTrain;
            configParam.pulseTrain_config.pulse_dir = pulse_dir;
            configParam.pulseTrain_config.pulse_len_msec = pulse_len_msec;
            configParam.pulseTrain_config.pulse_cnt = pulse_cnt;
            configParam.pulseTrain_config.time_bw_pulses_msec = time_bw_pulses_msec;            
            Main_Form.UpdatePinConfig(configParam);
        }

        public static void DutyCycleConfigUpdate(int time_period, int dutycycle)
        {
            updateConfigParam configParam = new updateConfigParam();
            Tuple<ePortName, ePinNum> pin;
            pin = Get_PortPin_Name();

            configParam.pName = (ePortName)pin.Item1;
            configParam.pNum = (ePinNum)pin.Item2;
            configParam.altFun = eAltFunction.eMeasDutyCycle;
            configParam.dutyCycle_config.time_period = time_period;
            configParam.dutyCycle_config.dutycycle = dutycycle;
            Main_Form.UpdatePinConfig(configParam);
        }        

        /// <summary>
        /// Updates the Alternate function combobox based on the selected direction
        /// </summary>
        /// <param name="direction"></param>
        public void UpdatePinAlternateFunction(eDirection direction)
        {
            string[] input_string = new string[] { s_gen_ip, s_pDetect_ip, s_EvntCntr_ip, s_QPulCntr_ip, s_PWHMeas_ip, s_FreqMeas_ip, s_PWMMeas_ip};

            string[] output_string = new string[] { s_gen_op, s_genSigPul_op, s_genPulTrn_op, s_genPWM_op, s_genMot_op};

            if (direction == eDirection.eInput)
            {
                cbAltFun.Items.Clear();
                cbAltFun.Items.AddRange(input_string);
                cbAltFun.SelectedIndex = 0;                
            }
            else if(direction == eDirection.eOutput)
            {
                cbAltFun.Items.Clear();
                cbAltFun.Items.AddRange(output_string);
                cbAltFun.SelectedIndex = 0;
            }
            else
            {
                //Unkown Direction 
            }
        }

        /// <summary>
        /// Clears the configure/monitor space to update new control
        /// </summary>
        private void clearPanel()
        {            
            panGenOutput.Visible = false;
            panPWMeasure.Visible = false;            
            panPWOp.Visible = false;
            panMotOp.Visible = false;
            panFreqMeasure.Visible = false;
            panGenInput.Visible = false;
            panDutyCycle.Visible = false;                        
        }

        /// <summary>
        /// Event handler for Alternate function selection combobox index changed
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void cbAltFun_SelectedIndexChanged(object sender, EventArgs e)
        {            
            ComboBox cmb = (ComboBox)sender;            
            string af_string = cmb.Text;


            last_changed_pin_name = cmb.Parent.Name;

            /************************ INPUT ************************/
            if (af_string == s_gen_ip)
            {
                // Clear existing panel and dispaly General input panel
                clearPanel();
                panGenInput.Visible = true;
                
                tbData.Text = "High";
            }       
            else if (af_string == s_pDetect_ip)
            {
                // Clear existing panel and dispaly new panel
                clearPanel();
                panMotOp.Visible = true;
                btMotorOpStart.Text = "Clear";

                // Display Pulse Detect conifguration window
                Winform_PulseDetectConfig wf = new Winform_PulseDetectConfig();                    
                wf.Show();                
            }
            else if (af_string == s_EvntCntr_ip)
            {
                // Clear existing panel and dispaly new panel
                clearPanel();
                panMotOp.Visible = true;
                btMotorOpStart.Text = "Reset";
            }
            else if (af_string == s_QPulCntr_ip)
            {
                // Clear existing panel and dispaly new panel
                clearPanel();
                // TODO: Display new panel for Quadrature pulse counter

                // Display Quadrature pulse counter configuration window
                var quad_rest_config_form = new Winform_QuadIpCntr_ResetConfig();                
                quad_rest_config_form.Show();                
            }
            else if (af_string == s_PWHMeas_ip)
            {
                // Clear existing panel and dispaly Pulse measure panel
                clearPanel();
                panPWMeasure.Visible = true;                
            }
            else if (af_string == s_FreqMeas_ip)
            {
                string[] freq_unit_string = new string[] { "Hz", "KHz", "MHz" };

                // Clear existing panel and dispaly Frequency measure panel
                clearPanel();
                panFreqMeasure.Visible = true;

                // Update the Frequ measure unit combobox
                cbFreqUnit.Items.Clear();
                cbFreqUnit.Items.AddRange(freq_unit_string);
                cbFreqUnit.SelectedIndex = 0;                
            }
            else if (af_string == s_PWMMeas_ip)
            {
                // Clear existing panel and dispaly Measure Duty cycle panel
                clearPanel();
                panDutyCycle.Visible = true;                            
            }            
            /************************ OUTPUT ************************/
            else if (af_string == s_gen_op)
            {
                // Clear existing panel and dispaly the general ouput panel
                clearPanel();                
                panGenOutput.Visible = true;
            }
            else if (af_string == s_genSigPul_op)
            {
                string[] time_unit_string = new string[] {"uSec", "mSec", "Sec"};

                // Clear existing panel and dispaly the PWM ouput panel
                clearPanel();
                panPWOp.Visible = true;

                cbPWUnit.Items.Clear();
                cbPWUnit.Items.AddRange(time_unit_string);
                cbPWUnit.SelectedIndex = 1;
            }

            else if (af_string == s_genPulTrn_op)
            {
                // Clear existing panel and dispaly the new panel
                clearPanel();
                panMotOp.Visible = true;
                btMotorOpStart.Text = "Start";

                // Display Pulse Train generation Configuration window
                Winform_PulseTrainConfig wf = new Winform_PulseTrainConfig();
                wf.Show();                                
            }
            else if (af_string == s_genPWM_op)
            {
                // Clear existing panel and dispaly the new panel
                clearPanel();
                panMotOp.Visible = true;
                btMotorOpStart.Text = "Start";

                // Display PWM Configuration window
                Winform_PWMGenConfig wf = new Winform_PWMGenConfig();
                wf.Show();                
            }
            else if (af_string ==s_genMot_op)
            {
                // Clear existing panel and dispaly the new panel
                clearPanel();
                panMotOp.Visible = true;

                // Display Motor Control Configuration window
                var config_form = new Winform_MotorCtrlConfig();                
                config_form.Show();                
            }
            else
            {                
                // Unknown selection, should not come here
                fLytPanAltFun.Controls.Clear();
            }            
        }

        /// <summary>
        /// Event handler for General Ouput conifguration
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void rbHigh_CheckedChanged(object sender, EventArgs e)
        {
            // Disable the event, this is to avoide multiple trigger of the events
            this.rbHigh.CheckedChanged -= new System.EventHandler(this.rbHigh_CheckedChanged);

            // General Output has changed the state
            last_changed_pin_name = fLytPanAltFun.Parent.Name;
            Tuple<ePortName, ePinNum> pin;
            pin = Get_PortPin_Name();

            if (rbHigh.Checked == true)
            {
                if (Main_Form.UpdateOutputState(pin.Item1, pin.Item2, true) != eStatus.eSuccess)
                {
                    rbHigh.Checked = false;
                    rbLow.Checked = true;
                }
            }
            else
            {
                if (Main_Form.UpdateOutputState(pin.Item1, pin.Item2, true) != eStatus.eSuccess)
                {
                    rbLow.Checked = false;
                    rbHigh.Checked = true;                    
                }
            }
            /* Re-enable the event */
            this.rbHigh.CheckedChanged += new System.EventHandler(this.rbHigh_CheckedChanged);
        }

        /// <summary>
        /// Event handler for motor o/p start click
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btMotorOpStart_Click(object sender, EventArgs e)
        {
            if (btMotorOpStart.Text == "Clear")
            {
                tbMotOpStatus.Text = "";
            } 
            else if (btMotorOpStart.Text == "Reset")
            {
                tbMotOpStatus.Text = "0";
                // todo
                //Reset the register value
            }
        }
        
        /// <summary>
        /// Event handler for Frequency unit changed
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void cbFreqUnit_SelectedIndexChanged(object sender, EventArgs e)
        {            
            if (tbFreq.Text != "")      // TODO: Needs modification in the logic
            {
                double data = Convert.ToDouble(tbFreq.Text);

                if (cbFreqUnit.SelectedIndex == 0)
                    tbFreq.Text = data.ToString();
                else if (cbFreqUnit.SelectedIndex == 1)
                {
                    data = data / 100;
                    tbFreq.Text = data.ToString();
                }
                else if (cbFreqUnit.SelectedIndex == 2)
                {
                    data = data / 1000;
                    tbFreq.Text = data.ToString();
                }
            }
        }

        /// <summary>
        /// Updates the user interface based on the input parameter
        /// </summary>
        /// <param name="param">paramter whihc needs to updated</param>
        /// <param name="data">data</param>
        public void UpdateInputValue(eConfigParam param, double data)
        {
            switch (param)
            {
                case eConfigParam.eGenInput:
                    if (data == 0)
                        tbData.Text = "Low";
                    else if (data == 1)
                        tbData.Text = "High";

                    break;

                case eConfigParam.eRisingEdge:
                    tbMotOpStatus.Text = "Rising Edge Detected";
                    break;

                case eConfigParam.eFallingEdge:
                    tbMotOpStatus.Text = "Falling Edge Detected";
                    break;

                case eConfigParam.eHighPulse:
                    tbMotOpStatus.Text = "High Pulse Detected";
                    break;

                case eConfigParam.eLowPulse:
                    tbMotOpStatus.Text = "Low Pulse Detected";
                    break;

                default:
                    break;
            }
        }        
    }
}
/***************************************************************************************************************/
/* End of File                                                                                                 */
/***************************************************************************************************************/